@@ -52,14 +52,17 @@
 #  signtool sign /v /f OSStudiosSPC.pfx "OpenShot Video Editor-2.0.0-win32.msi"
 
 import inspect
+import glob
 import os
 import sys
 import fnmatch
+import json
 from shutil import copytree, rmtree, copy
 from cx_Freeze import setup, Executable
 import cx_Freeze
 from PyQt5.QtCore import QLibraryInfo
 import shutil
+from installer.version_parser import parse_version_info, parse_build_name
 
 
 print (str(cx_Freeze))
@@ -76,6 +79,8 @@ python_packages = ["os",
                    "openshot",
                    "time",
                    "uuid",
+                   "idna",
+                   "sentry_sdk",
                    "shutil",
                    "threading",
                    "subprocess",
@@ -87,37 +92,56 @@ python_packages = ["os",
                    "requests",
                    "zmq",
                    "webbrowser",
-                   "json"
+                   "json",
                    ]
 
+# Modules to include
+python_modules = ["idna.idnadata",
+                  "sentry_sdk.integrations.stdlib",
+                  "sentry_sdk.integrations.excepthook",
+                  "sentry_sdk.integrations.dedupe",
+                  "sentry_sdk.integrations.atexit",
+                  "sentry_sdk.integrations.modules",
+                  "sentry_sdk.integrations.argv",
+                  "sentry_sdk.integrations.logging",
+                  "sentry_sdk.integrations.threading",
+                  ]
+
 # Determine absolute PATH of OpenShot folder
 PATH = os.path.dirname(os.path.realpath(__file__))  # Primary openshot folder
 
+# Look for optional --git-branch arg, and remove it
+git_branch_name = "develop"
+for arg in sys.argv:
+    if arg.startswith("--git-branch"):
+        sys.argv.remove(arg)
+        git_branch_name = arg.split("=")[-1].strip()
+
 # Make a copy of the src tree (temporary for naming reasons only)
+openshot_copy_path = os.path.join(PATH, "openshot_qt")
 if os.path.exists(os.path.join(PATH, "src")):
-    print("Copying modules to openshot_qt directory: %s" % os.path.join(PATH, "openshot_qt"))
+    print("Copying modules to openshot_qt directory: %s" % openshot_copy_path)
     # Only make a copy if the SRC directory is present (otherwise ignore this)
-    copytree(os.path.join(PATH, "src"), os.path.join(PATH, "openshot_qt"))
+    copytree(os.path.join(PATH, "src"), openshot_copy_path)
 
     # Make a copy of the launch.py script (to name it more appropriately)
     copy(os.path.join(PATH, "src", "launch.py"), os.path.join(PATH, "openshot_qt", "launch-openshot"))
 
-if os.path.exists(os.path.join(PATH, "openshot_qt")):
+if os.path.exists(openshot_copy_path):
     # Append path to system path
-    sys.path.append(os.path.join(PATH, "openshot_qt"))
-    print("Loaded modules from openshot_qt directory: %s" % os.path.join(PATH, "openshot_qt"))
+    sys.path.append(openshot_copy_path)
+    print("Loaded modules from openshot_qt directory: %s" % openshot_copy_path)
 
 # Append possible build server paths
 sys.path.insert(0, os.path.join(PATH, "build", "install-x86", "lib"))
+sys.path.insert(0, os.path.join(PATH, "build", "install-x86", "bin"))
 sys.path.insert(0, os.path.join(PATH, "build", "install-x64", "lib"))
-
+sys.path.insert(0, os.path.join(PATH, "build", "install-x64", "bin"))
 
 from classes import info
 from classes.logger import log
-
 log.info("Execution path: %s" % os.path.abspath(__file__))
 
-
 # Find files matching patterns
 def find_files(directory, patterns):
     """ Recursively find all files in a folder tree """
@@ -152,15 +176,26 @@ if os.path.exists(qt_system_path):
         if (file.startswith("qt_") or file.startswith("qtbase_")) and file.endswith(".qm"):
             shutil.copyfile(os.path.join(qt_system_path, file), os.path.join(qt_local_path, file))
 
-# Copy git log files into src files (if found)
-for project in ["libopenshot-audio", "libopenshot", "openshot-qt"]:
-    git_log_path = os.path.join(PATH, "build", "install-x64", "share", "%s.log" % project)
-    if os.path.exists(git_log_path):
-        src_files.append((git_log_path, "settings/%s.log" % project))
-    else:
-        git_log_path = os.path.join(PATH, "build", "install-x86", "share", "%s.log" % project)
-        if os.path.exists(git_log_path):
-            src_files.append((git_log_path, "settings/%s.log" % project))
+# Copy git log files into src/settings files (if found)
+version_info = {}
+for share_name in ["install-x64", "install-x86"]:
+    share_path = os.path.join(PATH, "build", share_name, "share")
+    if os.path.exists(share_path):
+        for git_log_filename in os.listdir(share_path):
+            git_log_filepath = os.path.join(share_path, git_log_filename)
+            if os.path.isfile(git_log_filepath):
+                src_files.append((git_log_filepath, "settings/%s" % git_log_filename))
+                if os.path.splitext(git_log_filepath)[1] == ".env":
+                    # No extension, parse version info
+                    version_info.update(parse_version_info(git_log_filepath))
+
+# If version info found (create src/settings/version.json file)
+if version_info:
+    # Calculate build name from version info
+    version_info["build_name"] = parse_build_name(version_info, git_branch_name)
+    version_path = os.path.join(openshot_copy_path, "settings", "version.json")
+    with open(version_path, "w") as f:
+        f.write(json.dumps(version_info, indent=4))
 
 if sys.platform == "win32":
     # Define alternate terminal-based executable
@@ -178,7 +213,11 @@ if sys.platform == "win32":
     src_files.append((os.path.join(PATH, "installer", "launch-win.bat"), "launch-win.bat"))
 
     # Add additional package
-    python_packages.append('idna')
+    python_packages.extend([
+        "idna",
+        "OpenGL",
+        "OpenGL_accelerate",
+    ])
 
     # Manually add zmq dependency (windows does not freeze it correctly)
     import zmq
@@ -187,6 +226,11 @@ if sys.platform == "win32":
     for filename in find_files(zmq_path, ["*"]):
         src_files.append((filename, os.path.join("lib", "zmq", os.path.relpath(filename, start=zmq_path))))
 
+    # Append all source files
+    src_files.append((os.path.join(PATH, "installer", "qt.conf"), "qt.conf"))
+    for filename in find_files("openshot_qt", ["*"]):
+        src_files.append((filename, os.path.join(os.path.relpath(filename, start=openshot_copy_path))))
+
 elif sys.platform == "linux":
     # Find libopenshot.so path (GitLab copies artifacts into local build/install folder)
     libopenshot_path = os.path.join(PATH, "build", "install-x64", "lib")
@@ -206,6 +250,29 @@ elif sys.platform == "linux":
     if os.path.exists(resvg_path):
         external_so_files.append((resvg_path, os.path.basename(resvg_path)))
 
+    # Add QtWebEngineProcess (if found)
+    web_process_path = ARCHLIB + "qt5/libexec/QtWebEngineProcess"
+    if os.path.exists(web_process_path):
+        external_so_files.append(
+            (web_process_path, os.path.basename(web_process_path)))
+
+    # Add QtWebEngineProcess Resources & Local
+    qt5_path = "/usr/share/qt5/"
+    for filename in find_files(os.path.join(qt5_path, "resources"), ["*"]):
+        external_so_files.append((filename, os.path.relpath(filename, start=qt5_path)))
+    for filename in find_files(os.path.join(qt5_path, "translations", "qtwebengine_locales"), ["*"]):
+        external_so_files.append((filename, os.path.relpath(filename, start=qt5_path)))
+
+    # Add Qt xcbglintegrations plugin
+    xcbgl_path = ARCHLIB + "qt5/"
+    for filename in find_files(os.path.join(xcbgl_path, "plugins", "xcbglintegrations"), ["*"]):
+        external_so_files.append((filename, os.path.relpath(filename, start=xcbgl_path)))
+
+    # Add libsoftokn3
+    nss_path = ARCHLIB + "nss/"
+    for filename in find_files(nss_path, ["*"]):
+        external_so_files.append((filename, os.path.basename(filename)))
+
     # Append Linux ICON file
     iconFile += ".svg"
     src_files.append((os.path.join(PATH, "xdg", iconFile), iconFile))
@@ -219,15 +286,32 @@ elif sys.platform == "linux":
     # Get a list of all openshot.so dependencies (scan these libraries for their dependencies)
     pyqt5_mod_files = []
     from importlib import import_module
-    for submod in ['Qt', 'QtWebKit', 'QtSvg', 'QtWebKitWidgets', 'QtWidgets', 'QtCore', 'QtGui', 'QtDBus']:
+    for submod in ['Qt', 'QtSvg', 'QtWidgets', 'QtCore', 'QtGui', 'QtDBus']:
         mod_name  = "PyQt5.{}".format(submod)
         mod = import_module(mod_name)
         pyqt5_mod_files.append(inspect.getfile(mod))
+    # Optional additions
+    for mod_name in [
+            'PyQt5.QtWebEngine',
+            'PyQt5.QtWebEngineWidgets',
+            'PyQt5.QtWebKit',
+            'PyQt5.QtWebKitWidgets',
+            ]:
+        try:
+            mod = import_module(mod_name)
+            pyqt5_mod_files.append(inspect.getfile(mod))
+        except ImportError as ex:
+            log.warning("Skipping {}: {}".format(mod_name, ex))
 
-    lib_list = [os.path.join(libopenshot_path, "libopenshot.so"),
-                "/usr/local/lib/libresvg.so",
-                ARCHLIB + "qt5/plugins/platforms/libqxcb.so"
-                ] + pyqt5_mod_files
+
+    lib_list = pyqt5_mod_files
+    for lib_name in [
+            os.path.join(libopenshot_path, "libopenshot.so"),
+            "/usr/local/lib/libresvg.so",
+            ARCHLIB + "qt5/plugins/platforms/libqxcb.so"
+            ]:
+        if os.path.exists(lib_name):
+            lib_list.append(lib_name)
 
     import subprocess
     for library in lib_list:
@@ -255,6 +339,7 @@ elif sys.platform == "linux":
             libpath = libdetailsparts[0].strip()
             libpath_file = os.path.basename(libpath)
             if (libpath
+                and os.path.exists(libpath)
                 and not libpath.startswith("/lib")
                 and "libnvidia-glcore.so" not in libpath
                 and libpath_file not in [
@@ -268,7 +353,13 @@ elif sys.platform == "linux":
                     "libICE.so.6",
                     "libp11-kit.so.0",
                     "libSM.so.6",
+                    # Next 5 are all part of glib2
+                    "libglib-2.0.so.0",
                     "libgobject-2.0.so.0",
+                    "libgio-2.0.so.0",
+                    "libgmodule-2.0.so.0",
+                    "libgthread-2.0.so.0",
+
                     "libdrm.so.2",
                     "libfreetype.so.6",
                     "libfontconfig.so.1",
@@ -278,59 +369,82 @@ elif sys.platform == "linux":
                     "libpangoft2-1.0.so.0",
                     "libharfbuzz.so.0",
                     "libthai.so.0",
-                ]
-                and not libpath_file.startswith("libxcb-")
-                ) \
-               or libpath_file in ["libgcrypt.so.11", "libQt5DBus.so.5", "libpng12.so.0", "libbz2.so.1.0", "libqxcb.so"]:
+                    ]
+               ) or libpath_file in [
+                    "libgcrypt.so.11",
+                    "libQt5DBus.so.5",
+                    "libpng12.so.0",
+                    "libbz2.so.1.0",
+                    "libqxcb.so",
+                    "libxcb-xinerama.so.0",
+                    "libpcre.so.3",
+                    "libselinux.so.1",
+                    ]:
+                external_so_files.append((libpath, libpath_file))
+                # Any other lib deps that fail to meet the inclusion
+                # criteria above will be silently skipped over
 
-                # Ignore missing files
-                if os.path.exists(libpath):
-                    filepath, filename = os.path.split(libpath)
-                    external_so_files.append((libpath, filename))
-
-    # Manually add missing files (that were missed in the above step). These files are required
-    # for certain distros (like Fedora, openSUSE, Debian, etc...)
-    # Also add Glib related files (required for some distros)
-
-    for added_lib in [ARCHLIB + "libssl.so",
-                      ARCHLIB + "libcrypto.so",
-                      ARCHLIB + "libglib-2.0.so",
-                      ARCHLIB + "libgio-2.0.so",
-                      ARCHLIB + "libgmodule-2.0.so",
-                      ARCHLIB + "libthread-2.0.so"
-                      ]:
-        if os.path.exists(added_lib):
-            external_so_files.append((added_lib, os.path.basename(added_lib)))
-        else:
-            log.warning("{}: not found, skipping".format(added_lib))
+    # Append all source files
+    src_files.append((os.path.join(PATH, "installer", "qt.conf"), "qt.conf"))
+    for filename in find_files("openshot_qt", ["*"]):
+        src_files.append((filename, os.path.join(os.path.relpath(filename, start=openshot_copy_path))))
 
 elif sys.platform == "darwin":
     # Copy Mac specific files that cx_Freeze misses
-    # JPEG library
-    for filename in find_files("/usr/local/Cellar/jpeg/8d/lib", ["libjpeg.8.dylib"]):
-        external_so_files.append((filename, filename.replace("/usr/local/Cellar/jpeg/8d/lib/", "")))
-
-    # Add libresvg (if found)
-    resvg_path = "/usr/local/lib/libresvg.dylib"
+   # Add libresvg (if found)
+    resvg_path = "/usr/local/lib/librsvg-2.dylib"
     if os.path.exists(resvg_path):
         external_so_files.append((resvg_path, resvg_path.replace("/usr/local/lib/", "")))
 
     # Copy openshot.py Python bindings
-    src_files.append((os.path.join(PATH, "openshot.py"), "openshot.py"))
-    src_files.append((os.path.join(PATH, "installer", "launch-mac.sh"), "launch-mac.sh"))
+    src_files.append((os.path.join(PATH, "installer", "launch-mac"), "launch-mac"))
 
     # Append Mac ICON file
     iconFile += ".hqx"
     src_files.append((os.path.join(PATH, "xdg", iconFile), iconFile))
 
-# Append all source files
-src_files.append((os.path.join(PATH, "installer", "qt.conf"), "qt.conf"))
-for filename in find_files("openshot_qt", ["*"]):
-    src_files.append((filename, filename.replace("openshot_qt/", "").replace("openshot_qt\\", "")))
+    # Add QtWebEngineProcess (if found)
+    qt_install_path = "/usr/local/qt5.15.X/qt5.15/5.15.0/clang_64/"
+    qt_webengine_path = os.path.join(qt_install_path, "lib", "QtWebEngineCore.framework", "Versions", "5")
+    web_process_path = os.path.join(qt_webengine_path, "Helpers", "QtWebEngineProcess.app", "Contents", "MacOS", "QtWebEngineProcess")
+    web_core_path = os.path.join(qt_webengine_path, "QtWebEngineCore")
+    external_so_files.append((web_process_path, os.path.basename(web_process_path)))
+    external_so_files.append((web_core_path, os.path.basename(web_core_path)))
+
+    # Add QtWebEngineProcess Resources & Local
+    for filename in find_files(os.path.join(qt_webengine_path, "Resources"), ["*"]):
+        external_so_files.append((filename, os.path.relpath(filename, start=os.path.join(qt_webengine_path, "Resources"))))
+    for filename in find_files(os.path.join(qt_webengine_path, "Resources", "qtwebengine_locales"), ["*"]):
+        external_so_files.append((filename, os.path.relpath(filename, start=os.path.join(qt_webengine_path, "Resources"))))
+    for filename in find_files(os.path.join(qt_install_path, "plugins"), ["*"]):
+        relative_filepath = os.path.relpath(filename, start=os.path.join(qt_install_path, "plugins"))
+        plugin_name = os.path.dirname(relative_filepath)
+        if plugin_name in ["imageformats", "platforms"]:
+            external_so_files.append((filename, relative_filepath))
+
+    # Append all source files
+    src_files.append((os.path.join(PATH, "installer", "qt.conf"), "qt.conf"))
+    for filename in find_files("openshot_qt", ["*"]):
+        src_files.append((filename, os.path.join("lib", os.path.relpath(filename, start=openshot_copy_path))))
+
+    # Exclude gif library which crashes on Mac
+    build_exe_options["bin_excludes"] = ["/System/Library/Frameworks/ImageIO.framework/Versions/A/Resources/libGIF.dylib",
+                                         "/usr/local/opt/giflib/lib/libgif.dylib",
+                                         "/usr/local/opt/tesseract/lib/libtesseract.4.dylib",
+                                         "/usr/local/opt/leptonica/lib/liblept.5.dylib"]
 
 # Dependencies are automatically detected, but it might need fine tuning.
 build_exe_options["packages"] = python_packages
 build_exe_options["include_files"] = src_files + external_so_files
+build_exe_options["includes"] = python_modules
+build_exe_options["excludes"] = ["distutils",
+                                 "sentry_sdk.integrations.django",
+                                 "numpy",
+                                 "setuptools",
+                                 "tkinter",
+                                 "pydoc_data",
+                                 "pycparser",
+                                 "pkg_resources"]
 
 # Set options
 build_options["build_exe"] = build_exe_options
@@ -363,4 +477,39 @@ setup(name=info.PRODUCT_NAME,
 
 # Remove temporary folder (if SRC folder present)
 if os.path.exists(os.path.join(PATH, "src")):
-    rmtree(os.path.join(PATH, "openshot_qt"), True)
+    rmtree(openshot_copy_path, True)
+
+# Fix a few things on the frozen folder(s)
+if sys.platform == "darwin":
+    # Mac issues with frozen folder and *.app folder
+    # We need to rewrite many dependency paths and library IDs
+    from installer.fix_qt5_rpath import *
+    build_path = os.path.join(PATH, "build")
+    for frozen_path in os.listdir(build_path):
+            if frozen_path.startswith("exe"):
+                fix_rpath(os.path.join(build_path, frozen_path))
+            elif frozen_path.endswith(".app"):
+                fix_rpath(os.path.join(build_path, frozen_path, "Contents", "MacOS"))
+                print_min_versions(os.path.join(build_path, frozen_path, "Contents", "MacOS"))
+
+elif sys.platform == "linux":
+    # Linux issues with frozen folder
+    # We need to remove some excess folders/files that are unneeded bloat
+    build_path = os.path.join(PATH, "build")
+    for frozen_path in os.listdir(build_path):
+            if frozen_path.startswith("exe"):
+                paths = ["lib/openshot_qt/",
+                         "lib/*opencv*",
+                         "lib/libopenshot*",
+                         "translations/",
+                         "locales/",
+                         "libQt5WebKit.so.5"]
+                for path in paths:
+                    full_path = os.path.join(build_path, frozen_path, path)
+                    for remove_path in glob.glob(full_path):
+                        if os.path.isfile(remove_path):
+                            log.info("Removing unneeded file: %s" % remove_path)
+                            os.unlink(remove_path)
+                        elif os.path.isdir(remove_path):
+                            log.info("Removing unneeded folder: %s" % remove_path)
+                            rmtree(remove_path)
