@@ -27,8 +27,11 @@
 
 import os
 
-from PyQt5.QtCore import QMimeData, Qt
-from PyQt5.QtGui import *
+from PyQt5.QtCore import (
+    QObject, QMimeData, Qt, pyqtSignal,
+    QSortFilterProxyModel, QPersistentModelIndex, QItemSelectionModel,
+)
+from PyQt5.QtGui import QIcon, QStandardItemModel, QStandardItem
 from PyQt5.QtWidgets import QMessageBox
 import openshot  # Python module for libopenshot (required video editing module installed separately)
 
@@ -38,33 +41,58 @@ from classes.app import get_app
 
 import json
 
-class TransitionStandardItemModel(QStandardItemModel):
-    def __init__(self, parent=None):
-        QStandardItemModel.__init__(self)
+
+class TransitionFilterProxyModel(QSortFilterProxyModel):
+    """Proxy class used for sorting and filtering model data"""
+
+    def filterAcceptsRow(self, sourceRow, sourceParent):
+        """Filter for common transitions and text filter"""
+
+        if get_app().window.actionTransitionsShowCommon.isChecked():
+            # Fetch the group name
+            index = self.sourceModel().index(sourceRow, 2, sourceParent)  # group name column
+            group_name = self.sourceModel().data(index)  # group name (i.e. common)
+
+            # Fetch the transitions name
+            index = self.sourceModel().index(sourceRow, 0, sourceParent)  # transition name column
+            trans_name = self.sourceModel().data(index)  # transition name (i.e. Fade In)
+
+            # Return, if regExp match in displayed format.
+            return group_name == "common" and self.filterRegExp().indexIn(trans_name) >= 0
+
+        # Continue running built-in parent filter logic
+        return super(TransitionFilterProxyModel, self).filterAcceptsRow(sourceRow, sourceParent)
+
+    def lessThan(self, left, right):
+        """Sort with both group name and transition name"""
+        leftData = left.data(self.sortRole())  # Transition name (or other role data)
+        rightData = right.data(self.sortRole())
+        leftGroup = left.sibling(left.row(), 2).data()  # group column
+        rightGroup = right.sibling(right.row(), 2).data()
+
+        return leftGroup < rightGroup and leftData < rightData
 
     def mimeData(self, indexes):
         # Create MimeData for drag operation
         data = QMimeData()
 
-        # Get list of all selected file ids
-        files = []
-        for item in indexes:
-            selected_row = self.itemFromIndex(item).row()
-            files.append(self.item(selected_row, 3).text())
-        data.setText(json.dumps(files))
+        # Create list from requested transition indexes
+        items = [i.sibling(i.row(), 3).data() for i in indexes]
+        data.setText(json.dumps(items))
         data.setHtml("transition")
 
         # Return Mimedata
         return data
 
 
-class TransitionsModel():
+class TransitionsModel(QObject):
+    ModelRefreshed = pyqtSignal()
+
     def update_model(self, clear=True):
         log.info("updating transitions model.")
         app = get_app()
 
-        # Get window to check filters
-        win = app.window
+        # Translations
         _ = app._tr
 
         # Clear all items
@@ -79,12 +107,22 @@ class TransitionsModel():
         transitions_dir = os.path.join(info.PATH, "transitions")
         common_dir = os.path.join(transitions_dir, "common")
         extra_dir = os.path.join(transitions_dir, "extra")
-        transition_groups = [{"type": "common", "dir": common_dir, "files": os.listdir(common_dir)},
-                             {"type": "extra", "dir": extra_dir, "files": os.listdir(extra_dir)}]
+        transition_groups = [
+            {"type": "common",
+             "dir": common_dir,
+             "files": os.listdir(common_dir)},
+            {"type": "extra",
+             "dir": extra_dir,
+             "files": os.listdir(extra_dir)},
+        ]
 
         # Add optional user-defined transitions folder
         if (os.path.exists(info.TRANSITIONS_PATH) and os.listdir(info.TRANSITIONS_PATH)):
-            transition_groups.append({"type": "user", "dir": info.TRANSITIONS_PATH, "files": os.listdir(info.TRANSITIONS_PATH)})
+            transition_groups.append(
+                {"type": "user",
+                 "dir": info.TRANSITIONS_PATH,
+                 "files": os.listdir(info.TRANSITIONS_PATH)}
+            )
 
         for group in transition_groups:
             type = group["type"]
@@ -115,17 +153,8 @@ class TransitionsModel():
                 else:
                     trans_name = self.app._tr(trans_name)
 
-                if not win.actionTransitionsShowAll.isChecked():
-                    if win.actionTransitionsShowCommon.isChecked():
-                        if not type == "common":
-                            continue  # to next file, didn't match filter
-
-                if win.transitionsFilter.text() != "":
-                    if not win.transitionsFilter.text().lower() in trans_name.lower():
-                        continue
-
                 # Check for thumbnail path (in build-in cache)
-                thumb_path = os.path.join(info.IMAGES_PATH, "cache",  "{}.png".format(fileBaseName))
+                thumb_path = os.path.join(info.IMAGES_PATH, "cache", "{}.png".format(fileBaseName))
 
                 # Check built-in cache (if not found)
                 if not os.path.exists(thumb_path):
@@ -149,11 +178,11 @@ class TransitionsModel():
                         reader.Close()
                         clip.Close()
 
-                    except:
+                    except Exception:
                         # Handle exception
-                        log.info('Invalid transition image file: %s' % filename)
+                        log.debug('Invalid transition image file %s', filename, exc_info=1)
                         msg = QMessageBox()
-                        msg.setText(_("{} is not a valid image file.".format(filename)))
+                        msg.setText(_("{} is not a valid transition file.".format(filename)))
                         msg.exec_()
                         continue
 
@@ -164,6 +193,7 @@ class TransitionsModel():
                 col.setIcon(QIcon(thumb_path))
                 col.setText(trans_name)
                 col.setToolTip(trans_name)
+                col.setData(type)
                 col.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsDragEnabled)
                 row.append(col)
 
@@ -189,14 +219,46 @@ class TransitionsModel():
                 row.append(col)
 
                 # Append ROW to MODEL (if does not already exist in model)
-                if not path in self.model_paths:
+                if path not in self.model_paths:
                     self.model.appendRow(row)
-                    self.model_paths[path] = path
+                    self.model_paths[path] = QPersistentModelIndex(row[3].index())
+
+        # Emit signal when model is updated
+        self.ModelRefreshed.emit()
 
     def __init__(self, *args):
+        # Init QObject superclass
+        super().__init__(*args)
 
         # Create standard model
         self.app = get_app()
-        self.model = TransitionStandardItemModel()
+        self.model = QStandardItemModel()
         self.model.setColumnCount(4)
         self.model_paths = {}
+
+        # Create proxy model (for sorting and filtering)
+        self.proxy_model = TransitionFilterProxyModel()
+        self.proxy_model.setDynamicSortFilter(True)
+        self.proxy_model.setFilterCaseSensitivity(Qt.CaseInsensitive)
+        self.proxy_model.setSortCaseSensitivity(Qt.CaseSensitive)
+        self.proxy_model.setSourceModel(self.model)
+        self.proxy_model.setSortLocaleAware(True)
+
+        # Create selection model to share between views
+        self.selection_model = QItemSelectionModel(self.proxy_model)
+
+        # Attempt to load model testing interface, if requested
+        # (will only succeed with Qt 5.11+)
+        if info.MODEL_TEST:
+            try:
+                # Create model tester objects
+                from PyQt5.QtTest import QAbstractItemModelTester
+                self.model_tests = []
+                for m in [self.proxy_model, self.model]:
+                    self.model_tests.append(
+                        QAbstractItemModelTester(
+                            m, QAbstractItemModelTester.FailureReportingMode.Warning)
+                    )
+                log.info("Enabled {} model tests for transition data".format(len(self.model_tests)))
+            except ImportError:
+                pass
